@@ -26,7 +26,7 @@ pub enum Primitive<'a> {
     /// Draw a 9 patch spanning the bounds of a rectangle, multiplied by a color.
     Draw9(Patch, Rectangle, Color),
     /// Draw an image stretched to the bounds of a rectangle, multiplied by a color.
-    DrawImage(Image, Rectangle, Color),
+    DrawImage(ImageData, Rectangle, Color),
 }
 
 /// A color with red, green, blue and alpha components.
@@ -44,7 +44,7 @@ pub struct Color {
 
 /// Reference to an image loaded by the [`Ui`](../struct.Ui.html).
 #[derive(Clone, Debug)]
-pub struct Image {
+pub struct ImageData {
     /// The texture atlas identifier that this image resides in.
     pub texture: usize,
     pub(crate) cache_id: Arc<usize>,
@@ -58,7 +58,7 @@ pub struct Image {
 #[derive(Clone, Debug)]
 pub struct Patch {
     /// The `Image` this `Patch` operates on.
-    pub image: Image,
+    pub image: ImageData,
     /// Horizontally stretchable regions in the 9 patch image.
     /// Every element is a pair of begin and end of the stretchable region.
     /// Defined in relative coordinates: 0.0 is the left side of the image,
@@ -82,14 +82,14 @@ pub struct Patch {
 }
 
 /// Generic background definition
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub enum Background {
     /// Draw no background
     None,
     /// Draw a solid color
     Color(Color),
     /// Draw a stretched image multiplied by a color
-    Image(Image, Color),
+    Image(ImageData, Color),
     /// Draw a 9 patch image multiplied by a color
     Patch(Patch, Color),
 }
@@ -186,6 +186,16 @@ pub enum Command {
 }
 
 impl Color {
+    /// Returns an (r, g, b) color with an alpha value of 1.
+    pub fn rgb(r: f32, g: f32, b: f32) -> Self {
+        Self::rgba(r, g, b, 1.0)
+    }
+
+    /// Returns an (r, g, b, a) color.
+    pub fn rgba(r: f32, g: f32, b: f32, a: f32) -> Self {
+        Self { r, g, b, a }
+    }
+
     /// Returns the color white
     pub fn white() -> Color {
         Color {
@@ -241,6 +251,16 @@ impl Color {
         self.a = a;
         self
     }
+
+    /// Modifies a color by blending it towards another color
+    pub fn blend(mut self, other: Color, factor: f32) -> Self {
+        let inverse = 1.0 - factor;
+        self.r = self.r * inverse + other.r * factor;
+        self.g = self.g * inverse + other.g * factor;
+        self.b = self.b * inverse + other.b * factor;
+        self.a = self.a * inverse + other.a * factor;
+        self
+    }
 }
 
 impl Patch {
