@@ -3,18 +3,15 @@ use smallvec::smallvec;
 use crate::draw::Primitive;
 use crate::event::{Event, Key};
 use crate::layout::{Rectangle, Size};
-use crate::stylesheet::{StyleState, Stylesheet};
-use crate::widget::{Context, IntoNode, Node, StateVec, Widget};
+use crate::node::{GenericNode, IntoNode, Node};
+use crate::style::{StyleState, Stylesheet};
+use crate::widget::{Context, StateVec, Widget};
 
 /// Pick an item from a dropdown box
-pub struct Dropdown<'a, T> {
-    state: &'a mut State,
-    items: Vec<Item<'a, T>>,
-}
-
-struct Item<'a, T> {
-    node: Node<'a, T>,
-    on_select: Option<T>,
+pub struct Dropdown<'a, T, F> {
+    items: Vec<Node<'a, T>>,
+    default_selection: Option<usize>,
+    on_select: F,
 }
 
 /// State for [`Dropdown`](struct.Dropdown.html).
@@ -30,49 +27,63 @@ enum InnerState {
     Pressed { scroll: f32, hover_item: usize },
 }
 
-impl<'a, T: 'a> Dropdown<'a, T> {
-    /// Construct a new dropdown
-    pub fn new(state: &'a mut State) -> Self {
-        Self {
-            state,
-            items: Vec::new(),
-        }
-    }
-
+impl<'a, T: 'a, F> Dropdown<'a, T, F> {
     /// Set the default selected item
     pub fn default_selection(mut self, item_index: usize) -> Self {
-        self.state.selected_item = self.state.selected_item.or(Some(item_index.min(self.items.len() - 1)));
+        self.default_selection = Some(item_index);
         self
     }
 
+    /// Sets the on_select callback for the dropdown, which is called when an item is selected
+    pub fn on_select<N: Fn(usize) -> T>(self, on_select: N) -> Dropdown<'a, T, N> {
+        Dropdown {
+            items: self.items,
+            default_selection: self.default_selection,
+            on_select,
+        }
+    }
+
     /// Add an item to the dropdown.
-    pub fn push(mut self, item: impl IntoNode<'a, T>, on_select: T) -> Self {
-        self.items.push(Item {
-            node: item.into_node(),
-            on_select: Some(on_select),
-        });
+    pub fn push(mut self, item: impl IntoNode<'a, T>) -> Self {
+        self.items.push(item.into_node());
         self
     }
 
     /// Add multiple items to the dropdown.
-    pub fn extend(mut self, items: impl IntoIterator<Item = (impl IntoNode<'a, T>, T)>) -> Self {
-        self.items.extend(items.into_iter().map(|(item, on_select)| Item {
-            node: item.into_node(),
-            on_select: Some(on_select),
-        }));
+    pub fn extend(mut self, items: impl IntoIterator<Item = impl IntoNode<'a, T>>) -> Self {
+        self.items.extend(items.into_iter().map(IntoNode::into_node));
         self
     }
 }
 
-impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
+impl<'a, T: 'a> Default for Dropdown<'a, T, fn(usize) -> T> {
+    fn default() -> Self {
+        Self {
+            items: Vec::new(),
+            default_selection: None,
+            on_select: |_| panic!("on_select of `Dropdown` must be set"),
+        }
+    }
+}
+
+impl<'a, T: Send + 'a, F: Send + Fn(usize) -> T> Widget<'a, T> for Dropdown<'a, T, F> {
+    type State = State;
+
+    fn mount(&self) -> Self::State {
+        State {
+            selected_item: self.default_selection.map(|i| i.min(self.items.len() - 1)),
+            ..Default::default()
+        }
+    }
+
     fn widget(&self) -> &'static str {
         "dropdown"
     }
 
-    fn state(&self) -> StateVec {
-        match self.state.inner {
+    fn state(&self, state: &State) -> StateVec {
+        match state.inner {
             InnerState::Open { .. } | InnerState::Pressed { .. } => smallvec![StyleState::Open],
-            InnerState::Idle if self.state.hovered => smallvec![StyleState::Hover],
+            InnerState::Idle if state.hovered => smallvec![StyleState::Hover],
             InnerState::Idle => StateVec::new(),
         }
     }
@@ -81,22 +92,22 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
         self.items.len()
     }
 
-    fn visit_children(&mut self, visitor: &mut dyn FnMut(&mut Node<'a, T>)) {
+    fn visit_children(&mut self, visitor: &mut dyn FnMut(&mut dyn GenericNode<'a, T>)) {
         for item in self.items.iter_mut() {
-            visitor(&mut item.node);
+            visitor(&mut **item);
         }
     }
 
-    fn size(&self, style: &Stylesheet) -> (Size, Size) {
+    fn size(&self, _: &State, style: &Stylesheet) -> (Size, Size) {
         let width = match style.width {
-            Size::Shrink => Size::Exact(self.items.iter().fold(0.0f32, |size, item| match item.node.size().0 {
+            Size::Shrink => Size::Exact(self.items.iter().fold(0.0f32, |size, item| match item.size().0 {
                 Size::Exact(item_size) => size.max(item_size),
                 _ => size,
             })),
             other => other,
         };
         let height = match style.height {
-            Size::Shrink => Size::Exact(self.items.iter().fold(0.0f32, |size, item| match item.node.size().1 {
+            Size::Shrink => Size::Exact(self.items.iter().fold(0.0f32, |size, item| match item.size().1 {
                 Size::Exact(item_size) => size.max(item_size),
                 _ => size,
             })),
@@ -108,21 +119,29 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
             .resolve_size((style.width, style.height), (width, height), style.padding)
     }
 
-    fn hit(&self, layout: Rectangle, clip: Rectangle, _: &Stylesheet, x: f32, y: f32) -> bool {
-        self.focused() || (layout.point_inside(x, y) && clip.point_inside(x, y))
+    fn hit(&self, state: &State, layout: Rectangle, clip: Rectangle, _: &Stylesheet, x: f32, y: f32) -> bool {
+        self.focused(state) || (layout.point_inside(x, y) && clip.point_inside(x, y))
     }
 
-    fn focused(&self) -> bool {
-        matches!(self.state.inner, InnerState::Open { .. } | InnerState::Pressed { .. })
+    fn focused(&self, state: &State) -> bool {
+        matches!(state.inner, InnerState::Open { .. } | InnerState::Pressed { .. })
     }
 
-    fn event(&mut self, layout: Rectangle, clip: Rectangle, _: &Stylesheet, event: Event, context: &mut Context<T>) {
-        self.state.inner = match (event, std::mem::replace(&mut self.state.inner, InnerState::Idle)) {
+    fn event(
+        &mut self,
+        state: &mut State,
+        layout: Rectangle,
+        clip: Rectangle,
+        _: &Stylesheet,
+        event: Event,
+        context: &mut Context<T>,
+    ) {
+        state.inner = match (event, std::mem::replace(&mut state.inner, InnerState::Idle)) {
             (Event::Cursor(x, y), InnerState::Idle) => {
                 let hovered = layout.point_inside(x, y) && clip.point_inside(x, y);
-                if hovered != self.state.hovered {
+                if hovered != state.hovered {
                     context.redraw();
-                    self.state.hovered = hovered;
+                    state.hovered = hovered;
                 }
                 InnerState::Idle
             }
@@ -132,9 +151,9 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
                     && x < layout.right
                     && y >= layout.bottom
                     && y < layout.bottom + self.items.len() as f32 * layout.height();
-                if hovered != self.state.hovered {
+                if hovered != state.hovered {
                     context.redraw();
-                    self.state.hovered = hovered;
+                    state.hovered = hovered;
                 }
 
                 let new_hover_item =
@@ -156,15 +175,15 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
                     && x < layout.right
                     && y >= layout.bottom
                     && y < layout.bottom + self.items.len() as f32 * layout.height();
-                if hovered != self.state.hovered {
+                if hovered != state.hovered {
                     context.redraw();
-                    self.state.hovered = hovered;
+                    state.hovered = hovered;
                 }
 
                 let new_hover_item =
                     (((y - layout.bottom) / layout.height()).floor().max(0.0) as usize).min(self.items.len() - 1);
 
-                if new_hover_item != hover_item || !self.state.hovered {
+                if new_hover_item != hover_item || !state.hovered {
                     context.redraw();
                     InnerState::Open {
                         scroll,
@@ -176,7 +195,7 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
             }
 
             (Event::Press(Key::LeftMouseButton), InnerState::Idle) => {
-                if self.state.hovered {
+                if state.hovered {
                     context.redraw();
                     InnerState::Open {
                         scroll: 0.0,
@@ -189,7 +208,7 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
 
             (Event::Press(Key::LeftMouseButton), InnerState::Open { scroll, hover_item }) => {
                 context.redraw();
-                if self.state.hovered {
+                if state.hovered {
                     InnerState::Pressed { scroll, hover_item }
                 } else {
                     InnerState::Idle
@@ -198,8 +217,8 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
 
             (Event::Release(Key::LeftMouseButton), InnerState::Pressed { hover_item, .. }) => {
                 context.redraw();
-                self.state.selected_item.replace(hover_item);
-                context.extend(self.items[hover_item].on_select.take());
+                state.selected_item.replace(hover_item);
+                context.push((self.on_select)(hover_item));
                 InnerState::Idle
             }
 
@@ -207,49 +226,56 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
         };
     }
 
-    fn draw(&mut self, layout: Rectangle, clip: Rectangle, style: &Stylesheet) -> Vec<Primitive<'a>> {
+    fn draw(
+        &mut self,
+        state: &mut State,
+        layout: Rectangle,
+        clip: Rectangle,
+        style: &Stylesheet,
+    ) -> Vec<Primitive<'a>> {
         let content = style.background.content_rect(layout, style.padding);
-        let focused = self.focused();
+        let focused = self.focused(state);
 
         let mut result = Vec::new();
         if focused {
             result.push(Primitive::LayerUp);
         }
-        match self.state.inner {
+        match state.inner {
             InnerState::Idle => {
                 result.extend(style.background.render(layout));
-                if let Some(selected) = self.state.selected_item {
-                    result.extend(self.items[selected].node.draw(content, clip));
+                if let Some(selected) = state.selected_item {
+                    result.extend(self.items[selected].draw(content, clip));
                 }
             }
             InnerState::Open { hover_item, .. } | InnerState::Pressed { hover_item, .. } => {
+                let padding = style.background.padding();
                 let expanded = Rectangle {
                     left: layout.left,
                     top: layout.top,
                     right: layout.right,
-                    bottom: layout.bottom + self.items.len() as f32 * layout.height(),
+                    bottom: layout.bottom + self.items.len() as f32 * layout.height() + padding.top + padding.bottom,
                 };
                 result.extend(style.background.render(expanded));
                 for (index, item) in self.items.iter_mut().enumerate() {
                     if index == hover_item {
                         result.push(Primitive::DrawRect(
                             Rectangle {
-                                left: layout.left,
-                                top: layout.top + (1 + index) as f32 * layout.height(),
-                                right: layout.right,
-                                bottom: layout.bottom + (1 + index) as f32 * layout.height(),
+                                left: layout.left + padding.left,
+                                top: layout.top + (1 + index) as f32 * layout.height() + padding.top,
+                                right: layout.right - padding.right,
+                                bottom: layout.bottom + (1 + index) as f32 * layout.height() + padding.top,
                             },
                             style.color,
                         ));
                     }
 
                     let layout = Rectangle {
-                        left: content.left,
-                        top: content.top + (1 + index) as f32 * layout.height(),
-                        right: content.right,
+                        left: content.left + padding.left,
+                        top: content.top + (1 + index) as f32 * layout.height() + padding.top,
+                        right: content.right - padding.right,
                         bottom: content.bottom + (1 + index) as f32 * layout.height(),
                     };
-                    result.extend(item.node.draw(layout, clip));
+                    result.extend(item.draw(layout, clip));
                 }
             }
         }
@@ -260,9 +286,9 @@ impl<'a, T: Send + 'a> Widget<'a, T> for Dropdown<'a, T> {
     }
 }
 
-impl<'a, T: 'a + Send> IntoNode<'a, T> for Dropdown<'a, T> {
+impl<'a, T: 'a + Send, F: 'a + Send + Fn(usize) -> T> IntoNode<'a, T> for Dropdown<'a, T, F> {
     fn into_node(self) -> Node<'a, T> {
-        Node::new(self)
+        Node::from_widget(self)
     }
 }
 
